@@ -1,25 +1,32 @@
 import math
 
-def almostEqual(v1, v2, diff=0.0000000001):
+def almostEqual(v1, v2, diff=0.0001):
     return abs(v1-v2) < diff
 
-def assertAlmostEqual(v1, v2, diff=0.0000000001):
+def assertAlmostEqual(v1, v2, diff=0.0001):
     assert almostEqual(v1, v2, diff), '%f and %f were more than %f apart' % (v1, v2, diff)
 
 class Candidate(object):
     
-    def __init__(self, name):
+    def __init__(self, name, slate=None):
         self.name = name
+        self.slate = slate
+    
+    def __str__(self):
+        if self.slate:
+            return '{} ({})'.format(self.name, self.slate)
+        else:
+            return self.name
     
     def __repr__(self):
-        return self.name
+        return '{}({}, {})'.format(self.__class__, self.name, self.slate)
     
 class Ballot(object):
     
-    def __init__(self, candidates=[], weight=1.0):
+    def __init__(self, candidates=[], weight=1.0, original_candidates=None):
         assert weight >= 0
         self.candidates = list(candidates)
-        self.original_candidates = list(candidates)
+        self.original_candidates = list(original_candidates if original_candidates else candidates)
         self.weight = weight
     
     def reset_weight(self):
@@ -41,7 +48,7 @@ class Ballot(object):
         return self.__add__(other)
         
     def __repr__(self):
-        return '(%s):%s' % (self.weight, self.candidates)
+        return '{}({} with {})'.format(self.__class__, self.weight, self.original_candidates)
         
 class DroopQuota(object):
     
@@ -155,7 +162,7 @@ class Race(object):
             
             # Add candidate to elected with a copy of the ballots used to elect them at adjusted weights
             # Copy ballots
-            ballots_used = set(Ballot(candidates=list(b.candidates), weight=b.weight) for b in ballots)
+            ballots_used = set(Ballot(candidates=list(b.candidates), original_candidates=list(b.original_candidates), weight=b.weight) for b in ballots)
             # Adjust ballot weights
             for ballot in ballots_used:
                 gregory.adjust_to_elected_weight(ballot)
@@ -209,27 +216,22 @@ class Race(object):
         sorted_hopefuls.sort(reverse=True)
         return sorted_hopefuls
     
-    def next_candidates_to_elect(self, sorted_hopefuls):
-        '''Returns the list of candidates to elect next with their ballots, breaking any ties for the top, in the following format
-        (
-            [(candidate, ballots), ...],
-            {
-                'break_tie': {candidate, ...}
-            }
-            or
-            None
-        )
-        '''
-        # Get top sum
-        top_sum = sorted_hopefuls[0][0]
-        # Get candidates that match the top sum and have passed the quota
-        top_candidates = dict((candidate, ballots) for (sum_ballots, candidate, ballots) in sorted_hopefuls if sum_ballots >= self.quota and almostEqual(sum_ballots, top_sum))
-        # Make sure all candidates are accounted for in tie breakers
-        assert all(candidate in self.candidate_tie_breakers for candidate in top_candidates), 'All candidates must be in tie breaking list'
-        # Sort the candidates based on the order in the tie breakers
-        candidates_to_elect = [(candidate, top_candidates[candidate]) for candidate in self.candidate_tie_breakers if candidate in top_candidates]
-        # Return the list of candidates and ballots, and a break_tie action if it was needed
-        return (candidates_to_elect, {'break_tie': set(top_candidates.keys())} if len(top_candidates) > 1 else None)
+    def get_top_candidates(self, sorted_hopefuls):
+        sorted_hopefuls = list(sorted_hopefuls)
+        
+        for i in range(len(sorted_hopefuls)):
+            # Get top sum
+            top_sum = sorted_hopefuls[0][0]
+            # Get candidates that match the top sum
+            top_candidates = dict((candidate, (sum_ballots, candidate, ballots)) for (sum_ballots, candidate, ballots) in sorted_hopefuls if almostEqual(sum_ballots, top_sum))
+            # Make sure all candidates are accounted for in tie breakers
+            assert all(c in self.candidate_tie_breakers for c in top_candidates), 'All candidates must be in tie breaking list'
+            # Sort the candidates based on the order in the tie breakers
+            orig_len = len(sorted_hopefuls)
+            candidate = next(c for c in self.candidate_tie_breakers if c in top_candidates)
+            sorted_hopefuls.remove(top_candidates[candidate])
+            assert len(sorted_hopefuls) == orig_len - 1
+            yield top_candidates[candidate] + ({'break_tie': set(top_candidates.keys())} if len(top_candidates) > 1 else None,)
     
     def next_candidate_to_exclude(self, sorted_hopefuls):
         '''Returns the next candidate to be excluded with their ballots, breaking any ties for last, in the following format
@@ -245,13 +247,12 @@ class Race(object):
         # Get the lowest sum
         lowest_sum = sorted_hopefuls[-1][0]
         # Get candidates that match the lowest sum
-        lowest_candidates = dict((candidate, ballots) for (sum_ballots, candidate, ballots) in sorted_hopefuls if almostEqual(sum_ballots, lowest_sum))
+        lowest_candidates = dict((candidate, (sum_ballots, candidate, ballots)) for (sum_ballots, candidate, ballots) in sorted_hopefuls if almostEqual(sum_ballots, lowest_sum))
+        # Make sure all candidates are accounted for in tie breakers
+        assert all(c in self.candidate_tie_breakers for c in lowest_candidates), 'All candidates must be in tie breaking list'        
         # Find the first candidate in the tie breakers with the sum
-        for candidate in self.candidate_tie_breakers:
-            if candidate in lowest_candidates:
-                # Return the candidate and ballots, and a break_tie action if it was needed
-                return ((candidate, lowest_candidates[candidate]), {'break_tie': set(lowest_candidates.keys())} if len(lowest_candidates) > 1 else None)
-        assert False, 'All candidates must be in tie breaking list'
+        candidate = next(c for c in self.candidate_tie_breakers if c in lowest_candidates)
+        return lowest_candidates[candidate] + ({'break_tie': set(lowest_candidates.keys())} if len(lowest_candidates) > 1 else None,)
     
     def get_status(self):
         '''Gets the status of the race in the format
@@ -311,12 +312,11 @@ class Race(object):
         # others are elected while there is still room. 
         # This behavior should be reviewed for correctness.
         candidates_to_be_elected = []
-        (next_candidates_to_elect, break_tie) = self.next_candidates_to_elect(sorted_hopefuls)
-        if break_tie:
-            actions.append(break_tie)
-        for (candidate, ballots) in next_candidates_to_elect:
-            if len(self.elected)+len(candidates_to_be_elected) < self.num_seats:
+        for (sum_ballots, candidate, ballots, break_tie) in self.get_top_candidates(sorted_hopefuls):
+            if sum_ballots >= self.quota and len(self.elected)+len(candidates_to_be_elected) < self.num_seats:
                 self.hopefuls.pop(candidate)
+                if break_tie:
+                    actions.append(break_tie)
                 candidates_to_be_elected.append((candidate, ballots))
         assert len(candidates_to_be_elected) == len(sorted_hopefuls)-len(self.hopefuls)
         
@@ -326,10 +326,9 @@ class Race(object):
                 actions.append({'elect': candidate, 'transfer': self.elect(candidate, ballots)})
         else:
             # or exclude last place candidate
-            (next_candidate_to_exclude, break_tie) = self.next_candidate_to_exclude(sorted_hopefuls)
+            (sum_ballots, candidate, ballots, break_tie) = self.next_candidate_to_exclude(sorted_hopefuls)
             if break_tie:
                 actions.append(break_tie)
-            (candidate, ballots) = next_candidate_to_exclude
             self.hopefuls.pop(candidate)
             actions.append({'exclude': candidate, 'transfer': self.exclude(candidate, ballots)})
         
@@ -353,12 +352,16 @@ class Race(object):
         '''
         sorted_hopefuls = self.get_sorted_hopefuls()
         actions = []
+            
         self.hopefuls.clear()
-        for (sum_ballots, candidate, ballots) in sorted_hopefuls:
+        for (sum_ballots, candidate, ballots, break_tie) in self.get_top_candidates(sorted_hopefuls):
+            if break_tie:
+                actions.append(break_tie)
             if sum_ballots >= self.minimum and len(self.elected) < self.num_seats:
                 actions.append({'elect': candidate, 'transfer': self.elect(candidate, ballots)})
             else:
                 actions.append({'reject': candidate, 'transfer': self.reject(candidate, ballots)})
+            
         return actions
     
     def continue_electing(self):
@@ -420,4 +423,4 @@ class Race(object):
         assertAlmostEqual(self.get_total_ballot_tally(), original_ballot_tally)
         
         return {'elected': self.elected, 'exhausted': self.exhausted_ballots, 'rounds': self.rounds}
-    
\ No newline at end of file
+    
